
use seahorse::{App, Command, Context, Flag, FlagType};
use std::env;

use crate::ascii::c_ascii;
use crate::data::data_toml;
use crate::data::url;
use crate::data::w_cfg;
use data::Notify as Notify;

pub mod data;
pub mod refresh;
pub mod token;
pub mod notify;
pub mod notify_read;
pub mod reply;
pub mod reply_link;
pub mod ascii;

fn main() {
    let args: Vec<String> = env::args().collect();
    let app = App::new(env!("CARGO_PKG_NAME"))
        .command(
            Command::new("ai")
            .alias("a")
            .action(c_ascii_art)
            .flag(
                Flag::new("type", FlagType::Bool)
                .description("type flag")
                .alias("t"),
            )
        )
        .command(
            Command::new("token")
            .alias("t")
            .description("handle\n\t\t\t$ ai t syui.bsky.social -p password")
            .action(c_token)
            .flag(
                Flag::new("password", FlagType::String)
                .description("password flag")
                .alias("p"),
                )
            .flag(
                Flag::new("server", FlagType::String)
                .description("server flag")
                .alias("s"),
            )
        )
        .command(
            Command::new("refresh")
            .alias("r")
            .action(c_refresh)
            .flag(
                Flag::new("server", FlagType::String)
                .description("server flag")
                .alias("s"),
            )
        )
        .command(
            Command::new("notify")
            .alias("n")
            .action(c_notify),
            )
        .command(
            Command::new("bot")
            .alias("b")
            .action(c_bot),
            )
        ;
    app.run(args);
}

fn c_ascii_art(c: &Context) {
    c_ascii(c.bool_flag("type"));
}

fn token(c: &Context) {
    let m = c.args[0].to_string();
    let h = async {
        if let Ok(p) = c.string_flag("password") {
            if let Ok(s) = c.string_flag("server") {
                let res = token::post_request(m.to_string(), p.to_string(), s.to_string()).await;
                w_cfg(&s, &res)
            } else {
                let res = token::post_request(m.to_string(), p.to_string(), "bsky.social".to_string()).await;
                w_cfg(&"bsky.social", &res)
            }
        }
    };
    let res = tokio::runtime::Runtime::new().unwrap().block_on(h);
    return res
}

fn c_token(c: &Context) {
    token(c);
}

fn refresh(c: &Context) {
    let h = async {
        let res = refresh::post_request().await;
        if let Ok(s) = c.string_flag("server") {
            w_cfg(&s, &res)
        } else {
            w_cfg("bsky.social", &res)
        }
    };
    let res = tokio::runtime::Runtime::new().unwrap().block_on(h);
    return res
}

fn c_refresh(c: &Context) {
    refresh(c);
}

fn notify() {
    let h = async {
        let j = notify::get_request(100).await;
        println!("{}", j);
    };
    let res = tokio::runtime::Runtime::new().unwrap().block_on(h);
    return res
}

fn c_notify(_c: &Context) {
    notify();
}

pub fn char_c(i: String) -> String {
    let l = 250;
    let mut s = String::new();
    for ii in i.chars().enumerate() {
        match ii.0 {
            n if n > l.try_into().unwrap() => {break}
            _ => {s.push(ii.1)}
        }
    }
    return s
}

fn bot(_c: &Context) {
    let h = async {
        let notify = notify::get_request(100).await;
        let notify: Notify = serde_json::from_str(&notify).unwrap();
        let n = notify.notifications;
        let length = &n.len();
        let reason = &n[0].reason;
        let handle = &n[0].author.handle;
        let did = &n[0].author.did;
        let read = n[0].isRead;
        let cid = &n[0].cid;
        let uri = &n[0].uri;
        println!("{}", length);
        println!("{}", reason);
        println!("{}", handle);
        println!("{}", did);
        println!("{}", read);
        println!("{} {}", cid, uri);
    };
    let res = tokio::runtime::Runtime::new().unwrap().block_on(h);
    return res

}

fn c_bot(c: &Context) {
    loop {
        bot(c);
    }
}
